/* GnomENIUS Calculator
 * Copyright (C) 1997 George Lebl.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

%{
#include <gmp.h>
#include "eval.h"

extern tree_t *tstack[256/*MAX_STACK*/];
extern int tstackp;

#define PUSH_FUNC(ACT) t_push(makefuncb(ACT,tstack,&tstackp),tstack,&tstackp);

%}

%union {
	mpz_t ival;
	mpf_t fval;
}

%token <ival> NORMINT NEGNORMINT
%token <fval> FLOAT NEGFLOAT

%token INFIX_EXPR POSTFIX_EXPR PREFIX_EXPR NEG

%left '+' '-'
%left '*' '/'

%right NEG UMINUS '!'
%right '^'

%%

expr:		INFIX_EXPR inexpr
	|	POSTFIX_EXPR poexpr
	|	PREFIX_EXPR prexpr
	|	error { yyclearin; }
	;

inexpr:		inexpr '+' inexpr	{ PUSH_FUNC(E_PLUS); }
	|	inexpr '-' inexpr	{ PUSH_FUNC(E_MINUS); }
	|	inexpr '*' inexpr	{ PUSH_FUNC(E_MUL); }
	|	inexpr '/' inexpr	{ PUSH_FUNC(E_DIV); }
	|	inexpr '!'		{ PUSH_FUNC(E_FACT); }
	| 	NEG inexpr 		{ PUSH_FUNC(E_NEG); }
	|	'-' inexpr %prec UMINUS	{ PUSH_FUNC(E_NEG); }
	| 	inexpr '^' inexpr	{ PUSH_FUNC(E_EXP); }
	|	'(' inexpr ')'
	|	posnum
	;

poexpr:		poexpr poexpr '+'	{ PUSH_FUNC(E_PLUS); }
	|	poexpr poexpr '-'	{ PUSH_FUNC(E_MINUS); }
	|	poexpr poexpr '*'	{ PUSH_FUNC(E_MUL); }
	|	poexpr poexpr '/'	{ PUSH_FUNC(E_DIV); }
	|	poexpr NEG		{ PUSH_FUNC(E_NEG); }
	|	poexpr '!'		{ PUSH_FUNC(E_FACT); }
	|	poexpr poexpr '^'	{ PUSH_FUNC(E_EXP); }
	|	'(' poexpr ')'
	|	num
	;

prexpr:		'+' prexpr prexpr	{ PUSH_FUNC(E_PLUS); }
	|	'-' prexpr prexpr	{ PUSH_FUNC(E_MINUS); }
	|	'*' prexpr prexpr	{ PUSH_FUNC(E_MUL); }
	|	'/' prexpr prexpr	{ PUSH_FUNC(E_DIV); }
	|	NEG prexpr 		{ PUSH_FUNC(E_NEG); }
	|	'!' prexpr		{ PUSH_FUNC(E_FACT); }
	|	'^' prexpr prexpr	{ PUSH_FUNC(E_EXP); }
	|	'(' prexpr ')'
	|	num
	;

num:		posnum
	|	negnum
	;

posnum:		NORMINT	{
				t_push(makenum_z($1),tstack,&tstackp);
				mpz_clear($1);
			}
	|	FLOAT	{
				t_push(makenum_f($1)
					,tstack,&tstackp);
				mpf_clear($1);
			}
	;

negnum:		NEGNORMINT	{
				t_push(makenum_z($1),tstack,&tstackp);
				mpz_clear($1);
			}
	|	NEGFLOAT	{
				t_push(makenum_f($1),tstack,&tstackp);
				mpf_clear($1);
			}
	;

%%
